#!/usr/bin/env bash
# author: Oksana Riba Grognuz & Almut Lütge


{ # try
    source src/config.sh
} || { 
    echo "Warning, src/config.sh does not exist and may be required for some functions."
}

function list_dataset_files () {
    local prop="http://schema.org/hasPart"
    files=(`renku dataset ls --format json-ld | jq -r '.[] | .[] | .'\"$prop\"'? | select( . != null) | .[]."@id"'`)
    
    for one_file in ${files[@]}
    do
        echo data/${one_file##https://renkulab.io/blob/*/data/}
    done
    
    }


function list_activity_inputs () {

    local json_file=${1:-.renku/provenance.json}
    local inputs=(`cat $json_file | jq -r '.[] | ."http://www.w3.org/ns/prov#entity"? | select( . != null) | .[]."@id" | select(. | contains("data/"))'`)
    
    for input_file in ${inputs[@]}
    do
        echo data/${input_file##https://renkulab.io/blob/*/data/}
    done
}


function list_plan_inputs () {
    # output: All files of pattern $2(default data) listed as inputs for any existing workflow
    local json_file=${1:-.renku/provenance.json}
    local pattern=${2:-data}
    local prop="https://renkulab.io/plans/"
    local prop2="http://schema.org/defaultValue"
    
    cat $json_file | jq -r '.[] | select(.'\"@id\"' | contains('\"$prop\"') and contains("inputs")) | .'\"$prop2\"'? | .[]."@value" | select(. | contains('\"$pattern\"'))'
}


function list_plan_outputs () {
    # output: All files of pattern $2(default data) listed as outputs for any existing workflow
    local json_file=${1:-.renku/provenance.json}
    local pattern=${2:-data}
    local prop="https://renkulab.io/plans/"
    local prop2="http://schema.org/defaultValue"
    
    cat $json_file | jq -r '.[] | select(.'\"@id\"' | contains('\"$prop\"') and contains("outputs")) | .'\"$prop2\"'? | .[]."@value" | select(. | contains('\"$pattern\"'))'
}


function find_unique () {
    echo "$@" | tr ' ' '\n' | sort | uniq -u
}


function find_files_to_process(){
    local json_file=${1:-.renku/provenance.json}
    local pattern=`echo ${IN_PREFIX[@]} | tr ' ' '|'`
    local pattern_excl=`echo $(list_plan_outputs $json_file $OUT_PATH)`
    
    unprocessed_in=$(find_unique `list_dataset_files $json_file` `list_plan_inputs $json_file` \
    | egrep $pattern)
    find_unique $pattern_excl `find_unique $unprocessed_in $pattern_excl`
}


function find_datasets_to_process(){
    local json_file=${1:-.renku/provenance.json}
    find_files_to_process $json_file | cut -f2 -d '/' | sort | uniq
}


function run_command_on_unprocessed(){
    local command=$1
    local json_file=${2:-.renku/provenance.json}
    files=(`find_files_to_process $json_file`)
    for one_file in ${files[@]}
    do
        $command $one_file
    done
}


function get_input_file_path(){
    local data_name=$1
    local input_index=$2
    echo data/${data_name}/${IN_PREFIX[$input_index]}_*.${IN_EXT[$input_index]}
}


function parse_r_arguments(){
    # input: assoicative array with R arguments-values
    # output: string for R CMD
    
    local -n arr=$1
    local quoted_args=`for i in "${!arr[@]}"; do   printf "${i}=\"${arr[$i]}\"\n"; done`
    printf %b "--args $quoted_args"
}


function parse_r_inputs(){
    # prints input files from associative array with R arguments
    # based on associtative array keys defined in config.sh
    local -n arr=$1
    local r_input=`echo ${!arr[@]} ${!IN_PREFIX[@]} | tr ' ' '\n' | sort | uniq -d`
    
    for i in ${r_input}
    do
       echo "--input ${R_args[$i]}"
    done | tr '\n' ' '
}


function parse_r_parameter(){
    # prints parameter defaults from associative parameter array 
    # based on associtative parameter array defined in config.sh
    local -n arr=${1:-PARAM_AR}
    
    for i in ${!arr[@]}
    do
       echo "--set ${i}=${arr[$i]}"
    done | tr '\n' ' '
}